@@ -60,6 +60,7 @@ _settings.analytics_events = AnalyticsEventsSettings()
 _settings.analytics_events.transactions = AnalyticsEventsTransactionsSettings()
 
 _settings.log_file = os.environ.get('NEW_RELIC_LOG', None)
+_settings.audit_log_file = os.environ.get('NEW_RELIC_AUDIT_LOG', None)
 
 def _environ_as_bool(name, default=False):
     flag = os.environ.get(name, default)
@@ -117,6 +118,7 @@ else:
     _settings.log_level = logging.INFO
 
 _settings.license_key = os.environ.get('NEW_RELIC_LICENSE_KEY', None)
+_settings.api_key = os.environ.get('NEW_RELIC_API_KEY', None)
 
 _settings.ssl = _environ_as_bool('NEW_RELIC_SSL', True)
 
@@ -303,6 +305,7 @@ def global_settings_dump():
     # it again.
 
     del settings['license_key']
+    del settings['api_key']
 
     del settings['proxy_user']
     del settings['proxy_pass']
