@@ -28,7 +28,7 @@ class SlowSqlNode(_SlowSqlNode):
     @property
     def explain_plan(self):
         return self.statement.explain_plan(self.connect_params,
-                self.cursor_params, self.execute_params)
+                self.cursor_params, self.execute_params, self.sql_format)
 
 _DatabaseNode = namedtuple('_DatabaseNode',
         ['dbapi2_module',  'sql', 'children', 'start_time', 'end_time',
@@ -57,7 +57,7 @@ class DatabaseNode(_DatabaseNode):
     @property
     def explain_plan(self):
         return self.statement.explain_plan(self.connect_params,
-                self.cursor_params, self.execute_params)
+                self.cursor_params, self.execute_params, self.sql_format)
 
     def time_metrics(self, stats, root, parent):
         """Return a generator yielding the timed metrics for this
